; ==================
; VG Symmetriefinder
;
; BCC 24 Abgabe
;
; von Casiopaya
;
; Fragen werden gerne beantwortet.
;
; Funktion:
;   Achsen werden als Vektoren auf einem 2D-Koordinatensystem betrachtet
;   und iterativ mit allen mglichen Drehpunkten und Drehwinkeln verglichen.
;   Die Normale des Achsenvektors und ein Punkt auf der Achse beschreibt die Linie der Pixel, die gespiegelt
;   sein muss. Erkennung des prozentual besten Matchings. Erkennung
;   mehrerer (exakt gleich guter) Matchings.
;
; Zusatzfunktionen:
;   - Bild muss nicht quadratisch sein
;   - Bild darf grer als 100 x 100 sein
;   - Symmetrieachse muss nicht durch gegenberliegende Bildseiten gehen.
; ==================

; Quelldatei aus der Kommandozeile:
Global SOURCE$ = Trim(CommandLine$())

; MIN_PROP wird ggf aus dem Parameter ausgelesen, falls nicht gesetzt wird ein Standardwert 0.8 verwendet:
Global MIN_PROP# = 0.8	; Wenn eine Symmetrieachse nur zu weniger als diesem Wert (Anzeil 0..1) matched wird sie gar nicht als Symmetrieachse erkannt.
For i% = 1 To Len(SOURCE$)
  If Mid(SOURCE$, i%, 5) = "-min=" Then
    i% = i% + 5
    minStart% = i%
    Repeat
      If (i% > Len(SOURCE$)) Then
        MIN_PROP# = Mid(SOURCE$, minStart%, i% - minStart%)
        SOURCE$ = Trim(Mid(SOURCE$, 1,  minStart% -6))
        Exit
      ElseIf (Mid(SOURCE$, i%, 1) = " ") Then
        MIN_PROP# = Mid(SOURCE$, minStart%, i% - minStart%)
        SOURCE$ = Trim(Mid(SOURCE$, 1,  minStart% -6) +  Mid(SOURCE$, i%,  Len(SOURCE$) -1))
        Exit
      End If
    i% = i% + 1
    Forever
  End If
Next
;Print "'" + SOURCE$ + "', '" + MIN_PROP# + "'"

; Die Zieldatei wird ins selbe Verzeichnis wie die Quelldatei geschrieben:
Global TARGET$ = SOURCE$ + "_RE.png"


; Weitere Parameter (nicht per Kommandozeile einstellbar):
Global PROP_PRINT = False ; Printed die Matching-Hhe direkt auf das Ergebnisbild


; Grafik laden, anzeigen:
Graphics 320, 240, 32, 2
img% = LoadImage(SOURCE$)
DrawImage img, 0, 0
imgBuffer = ImageBuffer(img)

; Grafik in Integer-Array laden:
width% = ImageWidth(img)
height% = ImageHeight(img)
Dim pixels%(height%, width%)
LockBuffer(imgBuffer)
For yPos% = 0 To height% - 1
  For xPos% = 0 To width% - 1
    If ReadPixelFast(xPos%, yPos%, imgBuffer) <> -1 Then
      pixels%(yPos%, xPos%) = 1
    End If
  Next
Next
UnlockBuffer (imgBuffer)


; Schrittgre wird abhngig vom Bild gewhlt, jedoch nicht kleiner 1.0:
Global ANGLE_STEP# = Float(width% + height%) / 100.0 ; Winkelschritt
If ANGLE_STEP# < 1.0 Then ANGLE_STEP# = 1.0 ; Rasterschritt
Global LINESTEP# = ANGLE_STEP# ; Durch Probieren hat sich herausgestellt, dass Winkelschritt und Rasterschritt gleich gro sein knnen.


; Array mit allen Steigungen (die pro Rotationspunkt geprft werden sollen = 0..180 Grad) fllen:
angleCount% = 180.0 / ANGLE_STEP#
Dim gradients#(angleCount% -1)
alpha# = 0
For anglePos% = 0 To angleCount% -1 
  gradients#(anglePos%) = - Tan(alpha#)
  alpha# = alpha# + ANGLE_STEP#
Next


; Datentyp zum Speichern des aktuell besten Matchings.
; Ist als Datentyp modelliert, um mehrere "beste Matchings" (gem Toleranzspezifikation) zu erlauben.
Type Matching 
  Field maxMatchProp# = 0
  Field maxMatchTotalCount% = 0
  Field maxMatchP1X# = 0
  Field maxMatchP1Y# = 0
  Field maxMatchP2X# = 0
  Field maxMatchP2Y# = 0
End Type
Dim BestMatchings.Matching(angleCount% * height% * width%) ; Maximal mgliche Anzahl bester Matchings... vllig unrealistischer Wert, aber braucht eh keinen Speicherplatz... :-)

; 1. Matching initiieren, wird gleich durch den ersten Wert berschrieben:
BestMatchingsCount% = 1; 
BestMatchings(0) = New Matching 


; Rechte und Untere Grenze des Koordinatensystems angeben:
x# = Float(width%) - 1.0
y# = - (Float(height%) - 1.0)


; Jede Achse in jedem Winkel wird mit jedem Punkt aus dem 5. Neuntel des Bildes geprft:
For sY# = (-(Float(height%) / 3.0) * 2.0) + 1 To  (-Float(height%) / 3.0) + 1
For sX# = (Float(width%) / 3.0) To (Float(width%) / 3.0) * 2.0 - 1
For gradPos% = 0 To angleCount% - 1

  ; Aktuelle Steigung
  gradient# = gradients#(gradPos%)
  
  ; Absolutglied auflsen:
  c# = sY# - (gradient# * sX#)

  ; Schnitt mit Achse Y = 0:
  substendXUp# = -c# / gradient#
    
  ; Schnitt mit Achse y = -(height-1):
  substendXDown# = (y# - c#) / gradient#
 
  ; Schnitt mit Achse x = 0
  substendYLeft# = c#
  
  ; Schnitt mit Achse x = width-1
  substendYRight# = gradient# * x# + c#
  
  ; Berechnen der beiden Schnittpunkte P1 und P2:
  p1Setted = False
  p1X# = 0
  p1Y# = 0
  
  If substendXUp# >= 0 And substendXUp# <= x# Then
    p1Setted = True
    p1X# = substendXUp#
    p1Y# = 0
  End If

  If substendXDown# >= 0 And substendXDown# <= x# Then
    If Not p1Setted Then
      p1Setted = True
      p1X# = substendXDown#
      p1Y# = y#
    Else:
      p2X# = substendXDown#
      p2Y# = y#
      Goto lineCoordsReady
    End If
  End If

  If substendYLeft# <= 0 And substendYLeft# >= y# Then
    If Not p1Setted Then
      p1X# = 0
      p1Y# = substendYLeft#
    Else:
      p2X# = 0
      p2Y# = substendYLeft#
      Goto lineCoordsReady
    End If
  End If
  
  p2X# = x#
  p2Y# = substendYRight#
 


 ; Der Matchin-Wert der aktuellen Achse wird berechnet:
.lineCoordsReady ; Jaja ich weis... recht dirty. Muss aber schnell sein.
  
  ; Der Achsenvektor wird berechnet:
  vX# = p2X# - p1X#
  vY# = p2Y# - p1Y#
  length# =  Sqr(vX# * vX# + vY# * vY#)
  ; und normalisiert:
  vX# = vX# / length#
  vY# = vY# / length#
  
  ; Die Normale der Achse wird berechnet (bereits normalisiert)
  nX# = -vY# * lineStep#
  nY# = vX# * lineStep#

  ; Counter fr matchende Pixel fr diese Achse zurcksetzen:
  matchCount% = 0
  missmatchCount% = 0

  aktX# = p1X#
  aktY# = p1Y#
  stepSizeX# = vX# * lineStep#
  stepSizeY# = vY# * lineStep#

  ; Die Achse wird von Punkt P1 nach P2 "begangen", an jedem Zwischenpunkt werden die senkrecht zur Achse und durch den 
  ; aktuellen Punkt laufenden Pixel verglichen:
  stepCount% = length# / lineStep#
  For i% = 0 To stepCount% 
    aktX# = aktX# + stepSizeX# ; Momentane X-Position auf der Achse
    aktY# = aktY# + stepSizeY# ; " Y "
    
    n1X# = aktX# ; Aktuelle Vergleichspositon auf der 1. Seite der Achse
    n1Y# = aktY# ; ""
    n2X# = aktX# ; Aktuelle Vergleichspositon auf der 2. Seite der Achse
    n2Y# = aktY# ; ""
    Repeat
        n1X# = n1X# + nX#
        n1Y# = n1Y# + nY#       
        If (n1X# >= 0 And n1X# <= x#) And (n1Y# >= y# And n1Y# <= 0) Then ; Punkt auf 1. Seite liegt noch im Bild
          n2X# = n2X# - nX#
          n2Y# = n2Y# - nY#             
          If (n2X# >= 0 And n2X# <= x#) And (n2Y# >= y# And n2Y# <= 0) Then ; Punkt auf 2. Seite liegt auch noch im Bild, erst jetzt kann ein VErgleich durchgefhrt werden
            If pixels%(-n1Y#, n1X#) = pixels%(-n2Y#, n2X#)  Then
              matchCount% = matchCount% + 1 ; Pixel auf 1. und 2. Seite der Achse sind gleich, Treffer!
            Else:
              missmatchCount% = missmatchCount% +1 ; Pixel sind nicht gleich, Fehler zhlen.
            End If
          Else:
            Exit
          End If
        Else
          Exit
        End If
    Forever    
  Next

  ; Es wird geprft ob die aktuell getestete Achse ein neues bestes Matching ist:
  matchProp# = Float(matchCount%)  / Float(matchCount% + missmatchCount%) ; Verhltnis (Treffer / Gesamtanzahl verglichener Pixel)
  
  ; Matching-Verhltnis ist hher oder Verhltnis ist gleich, aber insgesamt wurde eine grere Flche gematched:
  If (matchProp# > BestMatchings(0)\maxMatchProp#) Or ((matchProp# = BestMatchings(0)\maxMatchProp#) And (BestMatchings(0)\maxMatchTotalCount% < (matchCount% + missmatchCount%))) Then
    ; Neues tatschlich bestes Matching wurde gefunden 
    BestMatchings(0)\maxMatchProp# = matchProp#
    BestMatchings(0)\maxMatchTotalCount% = matchCount% + missmatchCount%
    BestMatchings(0)\maxMatchP1X# = p1X#
    BestMatchings(0)\maxMatchP1Y# = p1Y#
    BestMatchings(0)\maxMatchP2X# = p2X#
    BestMatchings(0)\maxMatchP2Y# = p2Y#
  
    ; Anzahl der besten Matchings wird auf 1 zurckgesetzt:
    BestMatchingsCount% = 1
    
  ; Absolut identisches Matching wurde gefunden und der Liste hinzugefgt.
  ElseIf (matchProp# = BestMatchings(0)\maxMatchProp#) And (BestMatchings(0)\maxMatchTotalCount% = (matchCount% + missmatchCount%)) Then   
    BestMatchings(BestMatchingsCount%) = New Matching
    BestMatchings(BestMatchingsCount%)\maxMatchP1X# = p1X#
    BestMatchings(BestMatchingsCount%)\maxMatchP1Y# = p1Y#
    BestMatchings(BestMatchingsCount%)\maxMatchP2X# = p2X#
    BestMatchings(BestMatchingsCount%)\maxMatchP2Y# = p2Y#
    BestMatchingsCount% = BestMatchingsCount% + 1
  End If

Next
Next
Next


; Evt. Zeichnen der Symmetrieachsen, Speichern des Ergebnis-Bilds:
Color 255, 0, 0
If BestMatchings(0)\maxMatchProp# >= MIN_PROP# Then
  For i% = 0 To  BestMatchingsCount% -1 
    Line BestMatchings(i%)\maxMatchP1X#, -BestMatchings(i%)\maxMatchP1Y#, BestMatchings(i%)\maxMatchP2X#, -BestMatchings(i%)\maxMatchP2Y#
  Next
  If PROP_PRINT Then 
    Print  "Match:"
    Print  BestMatchings(0)\maxMatchProp#
  End If
Else 
   Print   "NO match"
End If
GrabImage img, 0, 0
SaveImage( img, TARGET$ )


; Entfernen, wenn Resultat nicht betrachtet werden soll.
;WaitKey()