Graphics 800, 600, 0, 2
SetBuffer BackBuffer()

Image = LoadImage( Replace( CommandLine(), Chr( 34 ), "" ) )

If Not Image Then RuntimeError "Image doesn't exist!"

Local ImgWidth = ImageWidth( Image ), ImgHeight = ImageHeight( Image )
Global PickedX2D, PickedY2D

Const SHAPE_CONTRAST = 80

Dim ImgData( 0, 0 )

T = MilliSecs()
PrepareImage( Image, ImgWidth, ImgHeight )
ProcessImage( Image, ImgWidth, ImgHeight )
T = ( MilliSecs() - T )

Color 255, 255, 255
Text 300, 0, "Vebrauchte Zeit: " + T + "ms"

DrawImage Image, 0, 0
Flip 0
WaitKey()
End

Function PrepareImage( Image, Width, Height )
	Dim ImgData( Width - 1, Height - 1 )
	
	LockBuffer ImageBuffer( Image )
	For X = 0 To Width - 1
		For Y = 0 To Height - 1
			RGB = ReadPixelFast( X, Y, ImageBuffer( Image ) ) And $FFFFFF
			ImgData( X, Y ) = Sgn( RGB )
			
			If RGB <> $FFFFFF And RGB <> 0 Then
				Coloured = True
				Exit
			EndIf
		Next
		
		If Coloured Then Exit
	Next
	
	If Coloured Then
		For Y = 0 To Height - 2
			For X = 0 To Width - 2
				RGB = ReadPixelFast( X, Y, ImageBuffer( Image ) ) And $FFFFFF
				
				RGB_Right = ReadPixelFast( X + 1, Y, ImageBuffer( Image ) ) And $FFFFFF
				
				Contrast_R = ( RGB Shr 16 ) - ( RGB_Right Shr 16 )
				Contrast_G = ( ( RGB Shr 8 ) And $FF ) - ( ( RGB_Right Shr 8 ) And $FF )
				Contrast_B = ( RGB And $FF ) - ( RGB_Right And $FF )
				
				Contrast = Abs( Contrast_R ) + Abs( Contrast_G ) + Abs( Contrast_B )
				If Contrast > SHAPE_CONTRAST Then ImgData( X, Y ) = True Else ImgData( X, Y ) = False
			Next
		Next
		
		
		For X = 0 To Width - 2
			For Y = 0 To Height - 2
				RGB = ReadPixelFast( X, Y, ImageBuffer( Image ) ) And $FFFFFF
				
				RGB_Right = ReadPixelFast( X, Y + 1, ImageBuffer( Image ) ) And $FFFFFF
				
				Contrast_R = ( RGB Shr 16 ) - ( RGB_Right Shr 16 )
				Contrast_G = ( ( RGB Shr 8 ) And $FF ) - ( ( RGB_Right Shr 8 ) And $FF )
				Contrast_B = ( RGB And $FF ) - ( RGB_Right And $FF )
				
				Contrast = Abs( Contrast_R ) + Abs( Contrast_G ) + Abs( Contrast_B )
				If Contrast > SHAPE_CONTRAST Then ImgData( X, Y ) = True
			Next
		Next
		UnlockBuffer ImageBuffer( Image )
		
		For X = 1 To Width - 2
			For Y = 1 To Height - 2
				If ImgData( X, Y ) Then ImgData( X, Y - 1 ) = True
			Next
		Next
		
		For Y = 1 To Height - 2
			For X = 1 To Width - 2
				If ImgData( X, Y ) Then ImgData( X - 1, Y ) = True
			Next
		Next
	EndIf
	
	UnlockBuffer ImageBuffer( Image )
End Function

Function ProcessImage( Image, Width, Height )
	Local CenterX# = 0, CenterY# = 0, PixelCount = 0
	Reference = ImgData( 0, 0 )
	For X = 0 To Width - 1
		For Y = 0 To Height - 1
			If ImgData( X, Y ) <> Reference Then
				CenterX# = CenterX# + X
				CenterY# = CenterY# + Y
				PixelCount = PixelCount + 1
			EndIf
		Next
	Next
	
	CenterX# = CenterX#/PixelCount
	CenterY# = CenterY#/PixelCount
	
	Local MinDiff = 100000, MinAxisX#, MinAxisY#
	For Y = 0 To Height - 1
		LinePick2D( 0, Y, 1, 0, Width, Height )
		
		VY# = CenterX# - PickedX2D
		VX# = PickedY2D - CenterY#
		
		Diff = MirrorImage( Width, Height, CenterX#, CenterY#, VX#, VY# )
		
		If Diff < MinDiff Then
			MinDiff = Diff
			MinAxisX# = VX#
			MinAxisY# = VY#
		EndIf
	Next
	
	For X = 0 To Width - 1
		LinePick2D( X, 0, 0, 1, Width, Height )
		
		VY# = CenterX# - PickedX2D
		VX# = PickedY2D - CenterY#
		
		Diff = MirrorImage( Width, Height, CenterX#, CenterY#, VX#, VY# )
		
		If Diff < MinDiff Then
			MinDiff = Diff
			MinAxisX# = VX#
			MinAxisY# = VY#
		EndIf
	Next
	
	Factor = Width + Height
	
	TFormNormal MinAxisX#, MinAxisY#, 0, 0, 0
	
	StartX = CenterX# - TFormedX()*Factor
	StartY = Ceil( CenterY# - TFormedY()*Factor )
	EndX = CenterX# + TFormedX()*Factor
	EndY = Floor( CenterY# + TFormedY()*Factor )
	
	SetBuffer ImageBuffer( Image )
	Color 255, 0, 0
	Line StartX, StartY, EndX, EndY
	SetBuffer BackBuffer()
End Function

Function MirrorImage( Width, Height, CenterX#, CenterY#, VX, VY )
	TFormNormal VY, -VX, 0, 0, 0
	
	NormalX# = TFormedX()
	NormalY# = TFormedY()
	
	TFormNormal VX, VY, 0, 0, 0
	
	AxisX# = TFormedX()
	AxisY# = TFormedY()
	
	Local ImageDiff = 0
	For X = 0 To Width - 1
		For Y = 0 To Height - 1
			D# = DistanceFromAxis( X, Y, CenterX#, CenterY#, AxisX#, AxisY# )
			
			NewX = X + D#*NormalX#*2
			NewY = Y + D#*NormalY#*2
			
			If NewX >= 0 And NewX < Width And NewY >= 0 And NewY < Height Then
				If ImgData( NewX, NewY ) <> ImgData( X, Y ) Then ImageDiff = ImageDiff + 1
			EndIf
		Next
	Next
	
	Return ImageDiff
End Function

Function DistanceFromAxis#( X, Y, AX, AY, VY#, VX# )	;Wichtig! VX und VY mssen bereits normiert sein
	VX# = -VX#
	W# = -( AX*VX# + AY*VY# )								;Dadurch ersparen wir uns die Division und sparen jede Menge Zeit.
	
	Return X*VX + Y*VY + W
End Function

Function LinePick2D( X, Y, VX, VY, Width, Height )
	StartValue = ImgData( X, Y )
	
	While X >= 0 And X < Height And Y >= 0 And Y < Height
		If ImgData( X, Y ) <> StartValue Then
			PickedX2D = X
			PickedY2D = Y
			Return
		EndIf
		
		X = X + VX
		Y = Y + VY
	Wend
End Function